*______________________________________________________________________________;
*                                                                              ;
* Bootstrap macro, that processes everything during a single randomization.    ;
*______________________________________________________________________________;
*                                                                              ;
* OPTIONS MPRINT MLOGIC;
  %MACRO HAZBOOT(IN=IN, OUTEARLY=OUTEARLY, OUTCONST=OUTCONST, OUTLATE=OUTLATE,
                 SEED=-1, RESAMPL=1000, FRACTION=1.0, SLS=0.07, SLE=0.05);
*______________________________________________________________________________;
*                                                                              ;
* DEFINITIONS OF CALLING ARGUMENTS:                         DEFAULT:           ;
*                                                                              ;
*   IN       = Input data set name                          IN                 ;
*   OUTEARLY = Output dataset name, early phase             OUTEARLY           ;
*   OUTCONST = Output dataset name, constant phase          OUTCONST           ;
*   OUTLATE  = Output data set name, late phase             OUTLATE            ;
*   SEED     = Randomization seed                           -1                 ;
*              Default uses time of day as the seed                            ;
*   RESAMPL  = Number of valid models to generate           1000               ;
*   FRACTION = Fractional size of bootstrap                 1.0 (100% of n)    ;
*              If < 1, then a random sample of the size                        ;
*              specified will be used.                                         ;
*   SLE      = Criteria for variable entry                  0.07               ;
*   SLS      = Criteria for variable retention              0.05               ;
*                                                                              ;
* IN ADDITION:  A MACRO CALLED HMMODEL IS USED:                                ;
*   HMMODEL  = Macro that defines the model and variables to be used in the    ;
*              bootstrap analysis.  This model needs the following standard    ;
*              PROC HAZARD statements:                                         ;
*      TIME time sas name (this may be interval and left censored as well)     ;
*      EVENT event sas name;                                                   ;
*      PARMS MUE etc MUC etc MUL etc as needed with all shaping parameters     ;
*            FIXed                                                             ;
*      EARLY variable list                                                     ;
*      CONSTANT variable list                                                  ;
*      LATE variable list                                                      ;
*______________________________________________________________________________;
*                                                                              ;
* Get data set size                                                            ;
  DATA &IN; SET &IN NOBS=NUM; _NOBS_=NUM;
  IF _N_=1 THEN CALL SYMPUT('DS_SIZE', _NOBS_); RUN;
* Loop for each resampling (generates one observation in each output data set) ;
  OPTIONS NONOTES;
  %GLOBAL HZRNOTES HAZRC;
  %LET HZRNOTES=1;
  %LET SAMPLE=0;
  %LET TSAMPLES=0;
  %LET SIZE=&DS_SIZE*&FRACTION;
  %PUT 'Data Set Size' &DS_SIZE &FRACTION &SIZE;
  %DO %WHILE(&SAMPLE<&RESAMPL);
     %LET TSAMPLES=%EVAL(&TSAMPLES+1);
*******************************************************************************;
* Resample the data with replacement to generate a data set the same size as   ;
* the original one, but with observations chosen at random from the original   ;
* data set, unless FRACTION is < 1.0                                           ;
     DATA BOOTSTRP;
     DO J=1 TO &SIZE;                               *  Observations loop       ;
       R=RANUNI(&SEED);                             *  Random number (0 to 1)  ;
       N=MAX(MIN(FLOOR(&DS_SIZE*R)+1,&DS_SIZE),1);  *  Observation pointer     ;
       SET &IN POINT=N;                             *  Get observation         ;
       OUTPUT;                                      *  Output observation      ;
     END;
     STOP;
     RUN;
*______________________________________________________________________________;
*                                                                              ;
* Stepwise analysis with no printout                                           ;
*                                                                              ;
* NOTE:  You should assemble your variables VERY carefully, preferably after   ;
*        you have analyzed the data "by hand."  That analysis likely identified;
*        variables that are poor candidates for a model (for example, are      ;
*        associated with a single event, or occur with exceedingly low         ;
*        frequency), and these should not be used (remember that this is an    ;
*        automated analysis, not one you can "watch" as it is constructed!)    ;
*                                                                              ;
  %HAZARD(
  PROC HAZARD DATA=BOOTSTRP NOCOV NOCOR CONDITION=14 OUTHAZ=OUTEST
       NOPRINT NOLOG;
       SELECTION SLE=&SLE SLS=&SLS NOQ;
       %HMMODEL;
  );
*______________________________________________________________________________;
*                                                                              ;
* Check to see if we have a model.  If not, do another sampling in its place   ;
  %IF &HAZRC=0 %THEN %DO;
  %LET SAMPLE=%EVAL(&SAMPLE+1);
*                                                                              ;
* Prepare a data set for estimates from each randomization                     ;
  DATA EST; SET OUTEST (KEEP=_NAME_ _EST_) NOBS=ESTS;
  IF _N_>14;
*******************************************************************************;
* First 14 observations are related to the structure of the model, not the risk;
* factors, so we can discard them.  What is left is the symmetrical information;
* for 3 phases, in order, from intercept through the variables contained in the;
* first third of the file.  Note that all variables in any phase appear in all ;
* phases, but with the coefficient set to zero for variables not in a specific ;
* phase.                                                                       ;
  NV3=(ESTS-14)/3;
  NMOD=1+INT((_N_-15)/NV3);
  IF _EST_=0.0 THEN _EST_=.;
* PROC PRINT DATA=EST;
*******************************************************************************;
* Generate a separate data set for early, constant, and late hazard phases.    ;
* This requires that the names of variables be carried over to constant and    ;
* late phases.                                                                 ;
  DATA ESTEARLY; SET EST (WHERE=(NMOD=1));
    DROP NV3 NMOD;
* PROC PRINT;
*******************************************************************************;
* Extract variable names and put these onto the individual data sets           ;
  DATA NAMES; SET ESTEARLY (KEEP=_NAME_);
* PROC PRINT;
  DATA ESTCONST; SET EST (WHERE=(NMOD=2));
    DROP _NAME_ NV3 NMOD;
* PROC PRINT;
    DATA ESTCONST; MERGE NAMES ESTCONST;
    IF _N_=1 THEN _NAME_='C0';
    IF _EST_=0.0 THEN _EST_=.;
* PROC PRINT;

  DATA ESTLATE; SET EST (WHERE=(NMOD=3));
    DROP _NAME_ NV3 NMOD;
* PROC PRINT;
    DATA ESTLATE; MERGE NAMES ESTLATE;
    IF _N_=1 THEN _NAME_='L0';
    IF _EST_=0.0 THEN _EST_=.;
* PROC PRINT;
*******************************************************************************;
* Transpose the data sets for storage.  A single observation will be created   ;
* consisting of the estimates for the variables that are now the variable names;
  PROC TRANSPOSE DATA=ESTEARLY OUT=ESTEARLY;
    DATA ESTEARLY; SET ESTEARLY; DROP _NAME_; RESAMPL=&SAMPLE;
*   PROC PRINT;
  PROC TRANSPOSE DATA=ESTCONST OUT=ESTCONST;
    DATA ESTCONST; SET ESTCONST; DROP _NAME_; RESAMPL=&SAMPLE;
*   PROC PRINT;
  PROC TRANSPOSE DATA=ESTLATE OUT=ESTLATE;
    DATA ESTLATE; SET ESTLATE; DROP _NAME_; RESAMPL=&SAMPLE;
*   PROC PRINT;
*******************************************************************************;
* Append (concatinate) this result to the output data set                      ;
    DATA &OUTEARLY; SET &OUTEARLY ESTEARLY;
*   PROC PRINT;
    DATA &OUTCONST; SET &OUTCONST ESTCONST;
*   PROC PRINT;
    DATA &OUTLATE; SET &OUTLATE ESTLATE;
*   PROC PRINT;
    PROC DELETE DATA=ESTEARLY ESTCONST ESTLATE NAMES EST OUTEST;
    %END;
  %END;
*******************************************************************************;
* Summary of bootstrapping                                                     ;
  OPTIONS NOTES;
  %PUT Total number of resamplings was &TSAMPLES to generate &RESAMPL models.;
  %MEND HAZBOOT;
*______________________________________________________________________________;
*                                                                              ;
